package com.example.android.dynamic;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import android.app.Activity;
import android.os.Bundle;
import android.os.Environment;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import dalvik.system.BaseDexClassLoader;
import dalvik.system.DexClassLoader;
import dalvik.system.DexFile;
import dalvik.system.PathClassLoader;

public class MainActivity extends Activity {
	private TextView mJarPath;
	private TextView mLogView;
	private String mDexPath;
	private File mDexOutputDir;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		mDexOutputDir = getDir("dex", MODE_PRIVATE);
		for (File file : mDexOutputDir.listFiles()) {
			file.delete();
		}

		Button button1 = (Button) findViewById(R.id.button1);
		Button button2 = (Button) findViewById(R.id.button2);
		Button button3 = (Button) findViewById(R.id.button3);
		Button button4 = (Button) findViewById(R.id.button4);
		Button button5 = (Button) findViewById(R.id.button5);
		Button convert = (Button) findViewById(R.id.convert);
		Button clear = (Button) findViewById(R.id.clear);
		mJarPath = (TextView) findViewById(R.id.textView1);
		mLogView = (TextView) findViewById(R.id.textView2);

		button1.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				ClassLoader loader = createClassLoader1(mDexPath);
				invoke(loader);
			}
		});

		button2.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				ClassLoader loader = createClassLoader2(mDexPath);
				invoke(loader);
			}
		});

		button3.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				ClassLoader loader = createClassLoader3(mDexPath);
				invoke(loader);
			}
		});

		button4.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				createClassLoader4(mDexPath);
			}
		});

		button5.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				ClassLoader loader = getClassLoader();
				invoke(loader);
			}
		});

		convert.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				String jarPath = Environment.getExternalStorageDirectory().getPath() + "/Hello.jar";
				File converted = new File(mDexOutputDir, "Hello_converted.jar");
				mDexPath = converted.getAbsolutePath();
				mJarPath.setText(mDexPath);
				if (converted.exists()) {
					append("Convert skipped");
				} else {
					String[] args = new String[] {
							"--dex",
							"--output=" + mDexPath,
							jarPath
					};
					try {
						Class<?> clazz = getClassLoader().loadClass("com.android.dx.command.Main");
						Method method = clazz.getMethod("main", String[].class);
						method.invoke(null, (Object) args);
						append("Convert finished");
					} catch (ClassNotFoundException e) {
						append(e.getMessage());
					} catch (NoSuchMethodException e) {
						append(e.getMessage());
					} catch (IllegalArgumentException e) {
						append(e.getMessage());
					} catch (IllegalAccessException e) {
						append(e.getMessage());
					} catch (InvocationTargetException e) {
						append(e.getMessage());
					}
				}
			}
		});

		clear.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				clear();
			}
		});

		clear();
	}

	private ClassLoader createClassLoader1(String dexPath) {
		return new BaseDexClassLoader(dexPath, mDexOutputDir, null, getClassLoader());
	}

	private ClassLoader createClassLoader2(String dexPath) {
		return new DexClassLoader(dexPath, mDexOutputDir.getAbsolutePath(), null, getClassLoader());
	}

	private ClassLoader createClassLoader3(String dexPath) {
		return new PathClassLoader(dexPath, getClass().getClassLoader());
	}

	private void createClassLoader4(String dexPath) {
		String filename = dexPath.substring(dexPath.lastIndexOf("/"));
		filename = filename.replace(".jar", ".dex");
		try {
			DexFile dexFile = DexFile.loadDex(dexPath, mDexOutputDir.getAbsolutePath() + filename, 0);
			Class<?> clazz = dexFile.loadClass("com.example.hello.Hello", getClassLoader());
			if (clazz != null) {
				invoke(clazz);
			} else {
				append("class is null");
			}
		} catch (IOException e) {
			append(e.getMessage());
		}
	}

	private void invoke(Object loaderOrClass) {
		ClassLoader loader = null;
		Class<?> clazz = null;
		if (loaderOrClass instanceof ClassLoader) {
			loader = (ClassLoader) loaderOrClass;
		} else {
			clazz = (Class<?>) loaderOrClass;
		}
		try {
			if (loader != null) {
				clazz = loader.loadClass("com.example.hello.Hello");
			}
			Object obj = clazz.newInstance();
			Method method = clazz.getMethod("sayHello");
			Object greeting = method.invoke(obj);
			append((String) greeting);
		} catch (ClassNotFoundException e) {
			append(e.getMessage());
		} catch (InstantiationException e) {
			append(e.getMessage());
		} catch (IllegalAccessException e) {
			append(e.getMessage());
		} catch (NoSuchMethodException e) {
			append(e.getMessage());
		} catch (IllegalArgumentException e) {
			append(e.getMessage());
		} catch (InvocationTargetException e) {
			append(e.getMessage());
		}
	}

	private void append(String msg) {
		String text = mLogView.getText().toString();
		if (text.length() != 0) {
			mLogView.setText(text + "\n" + msg);
		} else {
			mLogView.setText(msg);
		}
	}

	private void clear() {
		mLogView.setText("");
		mDexPath = Environment.getExternalStorageDirectory().getPath() + "/Hello_dex.jar";
		mJarPath.setText(mDexPath);
	}
}
